@@ -26,26 +26,33 @@ window.GoAccess = window.GoAccess || {
 		this.AppData   = (this.opts || {}).panelData || {}; // hold raw data
 		this.AppWSConn = (this.opts || {}).wsConnection || {}; // WebSocket connection
 		this.AppPrefs  = {
-			'theme': 'darkGray',
-			'perPage': 10,
+			'theme': 'darkBlue',
+			'perPage': 7,
+			'layout': 'horizontal',
 		};
+		this.AppPrefs = GoAccess.Util.merge(this.AppPrefs, this.opts.prefs);
 
-		var ls = JSON.parse(localStorage.getItem('AppPrefs'));
-		this.AppPrefs = GoAccess.Util.merge(this.AppPrefs, ls);
+		if (GoAccess.Util.hasLocalStorage()) {
+			var ls = JSON.parse(localStorage.getItem('AppPrefs'));
+			this.AppPrefs = GoAccess.Util.merge(this.AppPrefs, ls);
+		}
 		if (Object.keys(this.AppWSConn).length)
 			this.setWebSocket(this.AppWSConn);
+
 	},
 
 	getPanelUI: function (panel) {
 		return panel ? this.AppUIData[panel] : this.AppUIData;
 	},
 
-	getPrefs: function () {
-		return this.AppPrefs;
+	getPrefs: function (panel) {
+		return panel ? this.AppPrefs[panel] : this.AppPrefs;
 	},
 
 	setPrefs: function () {
-		localStorage.setItem('AppPrefs', JSON.stringify(GoAccess.getPrefs()));
+		if (GoAccess.Util.hasLocalStorage()) {
+			localStorage.setItem('AppPrefs', JSON.stringify(GoAccess.getPrefs()));
+		}
 	},
 
 	getPanelData: function (panel) {
@@ -53,8 +60,9 @@ window.GoAccess = window.GoAccess || {
 	},
 
 	setWebSocket: function (wsConn) {
-		var str = wsConn.url.indexOf(':') != -1 ? wsConn.url : String(wsConn.url + ':' + wsConn.port);
-		var socket = new WebSocket('ws://' + str);
+		var str = /^(wss?:\/\/)?[^\/]+:[0-9]{1,5}\//.test(wsConn.url + "/") ? wsConn.url : String(wsConn.url + ':' + wsConn.port);
+		str = !/^wss?:\/\//i.test(str) ? 'ws://' + str : str;
+		var socket = new WebSocket(str);
 		socket.onopen = function (event) {
 			GoAccess.Nav.WSOpen();
 		}.bind(this);
@@ -70,7 +78,7 @@ window.GoAccess = window.GoAccess || {
 
 // HELPERS
 GoAccess.Util = {
-	months: ["Jan", "Feb", "Ma", "Apr", "May", "Jun", "Jul","Aug", "Sep", "Oct", "Nov", "Dec"],
+	months: ["Jan", "Feb", "Mar", "Apr", "May", "Jun", "Jul","Aug", "Sep", "Oct", "Nov", "Dec"],
 
 	// Add all attributes of n to o
 	merge: function (o, n) {
@@ -138,12 +146,12 @@ GoAccess.Util = {
 	},
 
 	// Format field value to human readable
-	fmtValue: function (value, valueType) {
+	fmtValue: function (value, dataType) {
 		var val = 0;
-		if (!valueType)
+		if (!dataType)
 			val = value;
 
-		switch (valueType) {
+		switch (dataType) {
 		case 'utime':
 			val = this.utime2str(value);
 			break;
@@ -219,6 +227,16 @@ GoAccess.Util = {
 		}
 		return o;
 	},
+
+	hasLocalStorage: function () {
+		try {
+			localStorage.setItem('test', 'test');
+			localStorage.removeItem('test');
+			return true;
+		} catch(e) {
+			return false;
+		}
+	},
 };
 
 // OVERALL STATS
@@ -241,9 +259,10 @@ GoAccess.OverallStats = {
 
 		var box = document.createElement('div');
 		box.innerHTML = GoAccess.AppTpls.General.items.render({
+			'id': x,
 			'className': ui.items[x].className,
 			'label': ui.items[x].label,
-			'value': GoAccess.Util.fmtValue(data[x], ui.items[x].valueType),
+			'value': GoAccess.Util.fmtValue(data[x], ui.items[x].dataType),
 		});
 		row.appendChild(box);
 
@@ -320,11 +339,29 @@ GoAccess.Nav = {
 			}.bind(this);
 		}.bind(this));
 
+		$$('.layout-horizontal', function (item) {
+			item.onclick = function (e) {
+				this.setLayout('horizontal');
+			}.bind(this);
+		}.bind(this));
+
+		$$('.layout-vertical', function (item) {
+			item.onclick = function (e) {
+				this.setLayout('vertical');
+			}.bind(this);
+		}.bind(this));
+
 		$$('[data-perpage]', function (item) {
 			item.onclick = function (e) {
 				this.setPerPage(e);
 			}.bind(this);
 		}.bind(this));
+
+		$$('[data-show-tables]', function (item) {
+			item.onclick = function (e) {
+				this.toggleTables();
+			}.bind(this);
+		}.bind(this));
 	},
 
 	downloadJSON: function (e) {
@@ -334,6 +371,39 @@ GoAccess.Nav = {
 		targ.download = 'goaccess-' + +new Date() + '.json';
 	},
 
+	setLayout: function (layout) {
+		if ('horizontal' == layout) {
+			$('.container').classList.add('container-fluid');
+			$('.container').classList.remove('container');
+		} else if ('vertical' == layout) {
+			$('.container-fluid').classList.add('container');
+			$('.container').classList.remove('container-fluid');
+		}
+
+		GoAccess.AppPrefs['layout'] = layout;
+		GoAccess.setPrefs();
+
+		GoAccess.Panels.initialize();
+		GoAccess.Charts.initialize();
+		GoAccess.Tables.initialize();
+	},
+
+	toggleTables: function () {
+		var ui = GoAccess.getPanelUI();
+		var showTables = GoAccess.Tables.showTables();
+		Object.keys(ui).forEach(function (panel, idx) {
+			if (!GoAccess.Util.isPanelValid(panel))
+				ui[panel]['table'] = !showTables;
+		}.bind(this));
+
+		GoAccess.AppPrefs['showTables'] = !showTables;
+		GoAccess.setPrefs();
+
+		GoAccess.Panels.initialize();
+		GoAccess.Charts.initialize();
+		GoAccess.Tables.initialize();
+	},
+
 	setTheme: function (theme) {
 		if (!theme)
 			return;
@@ -367,8 +437,9 @@ GoAccess.Nav = {
 		case 'referrers'       : return 'external-link';
 		case 'referring_sites' : return 'external-link';
 		case 'keyphrases'      : return 'google';
-		case 'geolocation'     : return 'location-arrow';
 		case 'status_codes'    : return 'warning';
+		case 'remote_user'     : return 'users';
+		case 'geolocation'     : return 'map-marker';
 		default                : return 'pie-chart';
 		}
 	},
@@ -399,8 +470,12 @@ GoAccess.Nav = {
 		return GoAccess.AppPrefs.theme || 'darkGray';
 	},
 
+	getLayout: function () {
+		return GoAccess.AppPrefs.layout || 'horizontal';
+	},
+
 	getPerPage: function () {
-		return GoAccess.AppPrefs.perPage || 10;
+		return GoAccess.AppPrefs.perPage || 7;
 	},
 
 	// Render left-hand side navigation options.
@@ -408,6 +483,8 @@ GoAccess.Nav = {
 		var o = {};
 		o[this.getTheme()] = true;
 		o['perPage' + this.getPerPage()] = true;
+		o['showTables'] = GoAccess.Tables.showTables();
+		o[this.getLayout()] = true;
 
 		$('.nav-list').innerHTML = GoAccess.AppTpls.Nav.opts.render(o);
 		$('nav').classList.toggle('active');
@@ -459,24 +536,99 @@ GoAccess.Nav = {
 
 // RENDER PANELS
 GoAccess.Panels = {
+	events: function () {
+		$$('[data-toggle=dropdown]', function (item) {
+			item.onclick = function (e) {
+				this.openOpts(e.currentTarget);
+			}.bind(this);
+			item.onblur = function (e) {
+				this.closeOpts(e);
+			}.bind(this);
+		}.bind(this));
+
+		$$('[data-plot]', function (item) {
+			item.onclick = function (e) {
+				GoAccess.Charts.redrawChart(e.currentTarget);
+			}.bind(this);
+		}.bind(this));
+
+		$$('[data-chart-type]', function (item) {
+			item.onclick = function (e) {
+				GoAccess.Charts.setChartType(e.currentTarget);
+			}.bind(this);
+		}.bind(this));
+
+		$$('[data-metric]', function (item) {
+			item.onclick = function (e) {
+				GoAccess.Tables.toggleColumn(e.currentTarget);
+			}.bind(this);
+		}.bind(this));
+	},
+
+	openOpts: function (targ) {
+		var panel = targ.getAttribute('data-panel');
+		targ.parentElement.classList.toggle('open');
+		this.renderOpts(panel);
+	},
+
+	closeOpts: function (e) {
+		e.currentTarget.parentElement.classList.remove('open');
+		// Trigger the click event on the target if not opening another menu
+		if (e.relatedTarget && e.relatedTarget.getAttribute('data-toggle') !== 'dropdown')
+			e.relatedTarget.click();
+	},
+
+	setPlotSelection: function (ui, prefs) {
+		var chartType = ((prefs || {}).plot || {}).chartType || ui.plot[0].chartType;
+		var metric = ((prefs || {}).plot || {}).metric || ui.plot[0].className;
+
+		ui[chartType] = true;
+		for (var i = 0, len = ui.plot.length; i < len; ++i)
+			if (ui.plot[i].className == metric)
+				ui.plot[i]['selected'] = true;
+	},
+
+	setColSelection: function (items, prefs) {
+		var columns = (prefs || {}).columns || {};
+		for (var i = 0, len = items.length; i < len; ++i)
+			if ((items[i].key in columns) && columns[items[i].key]['hide'])
+				items[i]['hide'] = true;
+	},
+
+	setOpts: function (panel) {
+		var ui = JSON.parse(JSON.stringify(GoAccess.getPanelUI(panel))), prefs = GoAccess.getPrefs(panel);
+		this.setPlotSelection(ui, prefs);
+		this.setColSelection(ui.items, prefs);
+		return ui;
+	},
+
+	renderOpts: function (panel) {
+		$('.panel-opts-' + panel).innerHTML = GoAccess.AppTpls.Panels.opts.render(this.setOpts(panel));
+		this.events();
+	},
+
 	enablePrev: function (panel) {
-		var pagination = '#panel-' + panel + ' .pagination a';
-		$(pagination + '.panel-prev').parentNode.classList.remove('disabled');
+		var $pagination = $('#panel-' + panel + ' .pagination a.panel-prev');
+		if ($pagination)
+			$pagination.parentNode.classList.remove('disabled');
 	},
 
 	disablePrev: function (panel) {
-		var pagination = '#panel-' + panel + ' .pagination a';
-		$(pagination + '.panel-prev').parentNode.classList.add('disabled');
+		var $pagination = $('#panel-' + panel + ' .pagination a.panel-prev');
+		if ($pagination)
+			$pagination.parentNode.classList.add('disabled');
 	},
 
 	enableNext: function (panel) {
-		var pagination = '#panel-' + panel + ' .pagination a';
-		$(pagination + '.panel-next').parentNode.classList.remove('disabled');
+		var $pagination = $('#panel-' + panel + ' .pagination a.panel-next');
+		if ($pagination)
+			$pagination.parentNode.classList.remove('disabled');
 	},
 
 	disableNext: function (panel) {
-		var pagination = '#panel-' + panel + ' .pagination a';
-		$(pagination + '.panel-next').parentNode.classList.add('disabled');
+		var $pagination = $('#panel-' + panel + ' .pagination a.panel-next');
+		if ($pagination)
+			$pagination.parentNode.classList.add('disabled');
 	},
 
 	enablePagination: function (panel) {
@@ -501,64 +653,66 @@ GoAccess.Panels = {
 	},
 
 	// Render the given panel given a user interface definition.
-	renderPanel: function (panel, ui) {
+	renderPanel: function (panel, ui, row, idx) {
+		var wrap = $('.wrap-panels');
+		var every = GoAccess.AppPrefs['layout'] == 'horizontal' ? 2 : 1;
+		var perRow = GoAccess.AppPrefs['layout'] == 'horizontal' ? 6 : 12;
+
+		// create a new bootstrap row every one or two elements depending on
+		// the layout
+		if (idx % every == 0) {
+			row = document.createElement('div');
+			row.setAttribute('class', 'row' + (every == 2 ? ' equal' : ''));
+			wrap.appendChild(row);
+		}
+
 		var data = GoAccess.getPanelData(panel);
 		this.hasSubItems(ui, data.data);
+		GoAccess.Tables.hasTables(ui);
 
+		// set the number of columns based on current layout
+		var col = document.createElement('div');
+		col.setAttribute('class', 'col-md-' + perRow + ' wrap-panel');
+		row.appendChild(col);
+
+		// per panel wrapper
 		var box = document.createElement('div');
 		box.id = 'panel-' + panel;
 		box.innerHTML = GoAccess.AppTpls.Panels.wrap.render(ui);
-		$('.wrap-panels').appendChild(box);
+		col.appendChild(box);
 
 		// Remove pagination if not enough data for the given panel
 		if (data.data.length <= GoAccess.getPrefs().perPage)
 			this.disablePagination(panel);
 		GoAccess.Tables.renderThead(panel, ui);
+
+		return row;
 	},
 
 	// Iterate over all available panels and render each panel
 	// structure.
 	renderPanels: function () {
-		var ui = GoAccess.getPanelUI();
+		var ui = GoAccess.getPanelUI(), idx = 0, row = null;
+
+		$('.wrap-panels').innerHTML = '';
 		for (var panel in ui) {
 			if (GoAccess.Util.isPanelValid(panel))
 				continue;
 			// Render panel given a user interface definition
-			this.renderPanel(panel, ui[panel]);
+			row = this.renderPanel(panel, ui[panel], row, idx++);
 		}
 	},
 
 	initialize: function () {
 		this.renderPanels();
+		this.events();
 	}
 };
 
 // RENDER CHARTS
 GoAccess.Charts = {
-	events: function () {
-		$$('[data-plot]', function (item) {
-			item.onclick = function (e) {
-				this.redrawChart(e.currentTarget);
-			}.bind(this);
-		}.bind(this));
-
-		$$('[data-chart-type]', function (item) {
-			item.onclick = function (e) {
-				this.setChartType(e.currentTarget);
-			}.bind(this);
-		}.bind(this));
-	},
-
-	setChartType: function (targ) {
-		var panel = targ.getAttribute('data-panel');
-		var type = targ.getAttribute('data-chart-type');
-
-		GoAccess.Util.setProp(GoAccess.AppPrefs, panel + '.plot', {'chartType': type});
-		GoAccess.setPrefs();
-
-		var plotUI = GoAccess.Util.getProp(GoAccess.AppState, panel + '.plot');
-		// Extract data for the selected panel and process it
-		this.drawPlot(panel, plotUI, this.getPanelData(panel));
+	getMetricKeys: function (panel, key) {
+		return GoAccess.getPanelUI(panel)['items'].map(function(a) {return a[key];});
 	},
 
 	getPanelData: function (panel, data) {
@@ -594,6 +748,18 @@ GoAccess.Charts = {
 		GoAccess.AppCharts[panel] = chart;
 	},
 
+	setChartType: function (targ) {
+		var panel = targ.getAttribute('data-panel');
+		var type = targ.getAttribute('data-chart-type');
+
+		GoAccess.Util.setProp(GoAccess.AppPrefs, panel + '.plot.chartType', type);
+		GoAccess.setPrefs();
+
+		var plotUI = GoAccess.Util.getProp(GoAccess.AppState, panel + '.plot');
+		// Extract data for the selected panel and process it
+		this.drawPlot(panel, plotUI, this.getPanelData(panel));
+	},
+
 	// Redraw a chart upon selecting a metric.
 	redrawChart: function (targ) {
 		var plot = targ.getAttribute('data-plot');
@@ -601,6 +767,9 @@ GoAccess.Charts = {
 		var ui = GoAccess.getPanelUI(panel);
 		var plotUI = ui.plot;
 
+		GoAccess.Util.setProp(GoAccess.AppPrefs, panel + '.plot.metric', plot);
+		GoAccess.setPrefs();
+
 		// Iterate over plot user interface definition
 		for (var x in plotUI) {
 			if (!plotUI.hasOwnProperty(x) || plotUI[x].className != plot)
@@ -640,6 +809,15 @@ GoAccess.Charts = {
 		return null;
 	},
 
+	getXKey: function (datum, key) {
+		var arr = [];
+		if (typeof key === 'string')
+			return datum[key];
+		for (var prop in key)
+			arr.push(datum[key[prop]]);
+		return arr.join(' ');
+	},
+
 	getAreaSpline: function (panel, plotUI, data) {
 		var dualYaxis = plotUI['d3']['y1'];
 
@@ -649,15 +827,17 @@ GoAccess.Charts = {
 			y1: dualYaxis ? plotUI['d3']['y1'].label : ''
 		})
 		.x(function (d) {
+			if ((((plotUI || {}).d3 || {}).x || {}).key)
+				return this.getXKey(d, plotUI['d3']['x']['key']);
 			return d.data;;
-		})
+		}.bind(this))
 		.y0(function (d) {
 			return +d[plotUI['d3']['y0']['key']];
 		})
-		.width($("#chart-" + panel).offsetWidth)
+		.width($("#chart-" + panel).getBoundingClientRect().width)
 		.height(175)
 		.format({
-			x: (this.findUIItem(panel, 'data') || {}).valueType || null,
+			x: (this.findUIItem(panel, 'data') || {}).dataType || null,
 			y0: ((plotUI.d3 || {}).y0 || {}).format,
 			y1: ((plotUI.d3 || {}).y1 || {}).format,
 		})
@@ -679,15 +859,17 @@ GoAccess.Charts = {
 			y1: dualYaxis ? plotUI['d3']['y1'].label : ''
 		})
 		.x(function (d) {
+			if ((((plotUI || {}).d3 || {}).x || {}).key)
+				return this.getXKey(d, plotUI['d3']['x']['key']);
 			return d.data;;
-		})
+		}.bind(this))
 		.y0(function (d) {
 			return +d[plotUI['d3']['y0']['key']];
 		})
-		.width($("#chart-" + panel).offsetWidth)
+		.width($("#chart-" + panel).getBoundingClientRect().width)
 		.height(175)
 		.format({
-			x: (this.findUIItem(panel, 'data') || {}).valueType || null,
+			x: (this.findUIItem(panel, 'data') || {}).dataType || null,
 			y0: ((plotUI.d3 || {}).y0 || {}).format,
 			y1: ((plotUI.d3 || {}).y1 || {}).format,
 		})
@@ -700,12 +882,28 @@ GoAccess.Charts = {
 		return chart;
 	},
 
+	getChartType: function (panel) {
+		var ui = GoAccess.getPanelUI(panel);
+		if (!ui.plot.length)
+			return '';
+
+		return GoAccess.Util.getProp(GoAccess.getPrefs(), panel + '.plot.chartType') || ui.plot[0].chartType;
+	},
+
+	getPlotUI: function (panel, ui) {
+		var metric = GoAccess.Util.getProp(GoAccess.getPrefs(), panel + '.plot.metric');
+		if (!metric)
+			return ui.plot[0];
+		return ui.plot.filter(function(v) {
+			return v.className == metric;
+		})[0];
+	},
+
 	getChart: function (panel, plotUI, data) {
-		var chart = null, type = null;
-		type = GoAccess.Util.getProp(GoAccess.getPrefs(), panel + '.plot.chartType') || plotUI.chartType;
+		var chart = null;
 
 		// Render given its type
-		switch (type) {
+		switch (this.getChartType(panel)) {
 		case 'area-spline':
 			chart = this.getAreaSpline(panel, plotUI, data);
 			break;
@@ -727,7 +925,7 @@ GoAccess.Charts = {
 			if (!ui[panel].plot || !ui[panel].plot.length)
 				continue;
 
-			plotUI = ui[panel].plot[0];
+			plotUI = this.getPlotUI(panel, ui[panel]);
 			// set ui plot data
 			GoAccess.Util.setProp(GoAccess.AppState, panel + '.plot', plotUI);
 
@@ -763,7 +961,6 @@ GoAccess.Charts = {
 					.call(GoAccess.AppCharts[panel].width($("#chart-" + panel).offsetWidth));
 			});
 		});
-		this.events();
 	}
 };
 
@@ -806,6 +1003,23 @@ GoAccess.Tables = {
 		}.bind(this));
 	},
 
+	toggleColumn: function (targ) {
+		var panel = targ.getAttribute('data-panel');
+		var metric = targ.getAttribute('data-metric');
+
+		var columns = (GoAccess.getPrefs(panel) || {}).columns || {};
+		if (metric in columns)
+			delete columns[metric];
+		else
+			GoAccess.Util.setProp(columns, metric + '.hide', true);
+
+		GoAccess.Util.setProp(GoAccess.AppPrefs, panel + '.columns', columns);
+		GoAccess.setPrefs();
+
+		GoAccess.Tables.renderThead(panel, GoAccess.getPanelUI(panel));
+		GoAccess.Tables.renderFullTable(panel);
+	},
+
 	sortColumn: function (ele) {
 		var field = ele.getAttribute('data-key');
 		var order = ele.getAttribute('data-order');
@@ -949,17 +1163,33 @@ GoAccess.Tables = {
 		var max = (value || {}).max;
 		var min = (value || {}).min;
 
+		// use metaType if exist else fallback to dataType
+		var vtype = ui.metaType || ui.dataType;
 		var className = ui.className || '';
-		className += ui.valueType != 'string' ? 'text-right' : '';
+		className += ui.dataType != 'string' ? 'text-right' : '';
 		return {
 			'className': className,
-			'max'      : max != undefined ? GoAccess.Util.fmtValue(max, ui.valueType) : null,
-			'min'      : min != undefined ? GoAccess.Util.fmtValue(min, ui.valueType) : null,
-			'value'    : val != undefined ? GoAccess.Util.fmtValue(val, ui.valueType) : null,
+			'max'      : max != undefined ? GoAccess.Util.fmtValue(max, vtype) : null,
+			'min'      : min != undefined ? GoAccess.Util.fmtValue(min, vtype) : null,
+			'value'    : val != undefined ? GoAccess.Util.fmtValue(val, vtype) : null,
 			'title'    : ui.meta,
+			'label'    : ui.metaLabel || null,
 		}
 	},
 
+	hideColumn: function (panel, col) {
+		var columns = (GoAccess.getPrefs(panel) || {}).columns || {};
+		return ((col in columns) && columns[col]['hide']);
+	},
+
+	showTables: function () {
+		return ('showTables' in GoAccess.getPrefs()) ? GoAccess.getPrefs().showTables : true;
+	},
+
+	hasTables: function (ui) {
+		ui['table'] = GoAccess.Tables.showTables();
+	},
+
 	renderMetaRow: function (panel, ui) {
 		// find the table to set
 		var table = $('.table-' + panel + ' tbody.tbody-meta');
@@ -970,6 +1200,8 @@ GoAccess.Tables = {
 		var data = GoAccess.getPanelData(panel).metadata;
 		for (var i = 0; i < uiItems.length; ++i) {
 			var item = uiItems[i];
+			if (this.hideColumn(panel, item.key))
+				continue;
 			var value = data[item.key];
 			cells.push(this.getMetaCell(item, value))
 		}
@@ -987,6 +1219,8 @@ GoAccess.Tables = {
 		var out = [];
 		for (var i = 0; i < uiItems.length; ++i) {
 			var uiItem = uiItems[i];
+			if (this.hideColumn(panel, uiItem.key))
+				continue;
 			// Data for the current user interface property.
 			// e.g., dataItem = Object {count: 13949, percent: 5.63}
 			var dataItem = dataItems[uiItem.key];
@@ -1004,11 +1238,11 @@ GoAccess.Tables = {
 	// e.g., value = Object {count: 14351, percent: 5.79}
 	getObjectCell: function (panel, ui, value) {
 		var className = ui.className || '';
-		className += ui.valueType != 'string' ? 'text-right' : '';
+		className += ui.dataType != 'string' ? 'text-right' : '';
 		return {
 			'className': className,
 			'percent': GoAccess.Util.getPercent(value),
-			'value': GoAccess.Util.fmtValue(GoAccess.Util.getCount(value), ui.valueType)
+			'value': GoAccess.Util.fmtValue(GoAccess.Util.getCount(value), ui.dataType)
 		};
 	},
 
@@ -1089,60 +1323,83 @@ GoAccess.Tables = {
 		});
 	},
 
-	renderTable: function (panel, page, expanded) {
-		var dataItems = GoAccess.getPanelData(panel).data;
-		var ui = GoAccess.getPanelUI(panel);
-
+	togglePagination: function (panel, page, dataItems) {
 		GoAccess.Panels.enablePagination(panel);
 		// Diable pagination next button if last page is reached
 		if (page >= this.getTotalPages(dataItems))
 			GoAccess.Panels.disableNext(panel);
 		if (page <= 1)
 			GoAccess.Panels.disablePrev(panel);
+	},
 
+	renderTable: function (panel, page) {
+		var dataItems = GoAccess.getPanelData(panel).data;
+		var ui = GoAccess.getPanelUI(panel);
+
+		this.togglePagination(panel, page, dataItems);
 		// Render data rows
 		this.renderDataRows(panel, ui, dataItems, page);
 		this.events();
 	},
 
+	renderFullTable: function (panel) {
+		var ui = GoAccess.getPanelUI(panel), page = 0;
+		// panel's data
+		var data = GoAccess.getPanelData(panel);
+		// render meta data
+		if (data.hasOwnProperty('metadata'))
+			this.renderMetaRow(panel, ui);
+
+		// render actual data
+		if (data.hasOwnProperty('data')) {
+			page = this.getCurPage(panel);
+			this.togglePagination(panel, page, data.data);
+			this.renderDataRows(panel, ui, data.data, page);
+		}
+	},
+
 	// Iterate over all panels and determine which ones should contain
 	// a data table.
 	renderTables: function () {
 		var ui = GoAccess.getPanelUI();
 		for (var panel in ui) {
-			if (GoAccess.Util.isPanelValid(panel))
+			if (GoAccess.Util.isPanelValid(panel) || !this.showTables())
 				continue;
-
-			// panel's data
-			var data = GoAccess.getPanelData(panel);
-			// render meta data
-			if (data.hasOwnProperty('metadata'))
-				this.renderMetaRow(panel, ui[panel]);
-			// render actual data
-			if (data.hasOwnProperty('data')) {
-				this.renderDataRows(panel, ui[panel], data.data, this.getCurPage(panel));
-			}
+			this.renderFullTable(panel);
 		}
 	},
 
 	// Given a UI panel definition, make a copy of it and assign the sort
 	// fields to the template object to render
 	sort2Tpl: function (panel, ui) {
-		var uiClone = JSON.parse(JSON.stringify(ui));;
+		var uiClone = JSON.parse(JSON.stringify(ui)), out = [];
 		var sort = GoAccess.Util.getProp(GoAccess.AppState, panel + '.sort');
-		uiClone['items'].forEach(function (item) {
+
+		for (var i = 0, len = uiClone.items.length; i < len; ++i) {
+			var item = uiClone.items[i];
+			if (this.hideColumn(panel, item.key))
+				continue;
+
 			item['sort'] = false;
 			if (item.key == sort.field && sort.order) {
 				item['sort'] = true;
 				item[sort.order.toLowerCase()] = true;
 			}
-		});
+			out.push(item);
+		}
+		uiClone.items = out;
 
 		return uiClone;
 	},
 
 	renderThead: function (panel, ui) {
-		$('.table-' + panel + '>thead').innerHTML = GoAccess.AppTpls.Tables.head.render(this.sort2Tpl(panel, ui));
+		var $thead = $('.table-' + panel + '>thead'), $colgroup = $('.table-' + panel + '>colgroup');
+		if ($thead && $colgroup && this.showTables()) {
+			ui = this.sort2Tpl(panel, ui);
+
+			$thead.innerHTML = GoAccess.AppTpls.Tables.head.render(ui);
+			$colgroup.innerHTML = GoAccess.AppTpls.Tables.colgroup.render(ui);
+		}
 	},
 
 	initialize: function () {
@@ -1166,12 +1423,14 @@ GoAccess.App = {
 			},
 			'Panels': {
 				'wrap': this.tpl($('#tpl-panel').innerHTML),
+				'opts': this.tpl($('#tpl-panel-opts').innerHTML),
 			},
 			'General': {
 				'wrap': this.tpl($('#tpl-general').innerHTML),
 				'items': this.tpl($('#tpl-general-items').innerHTML),
 			},
 			'Tables': {
+				'colgroup': this.tpl($('#tpl-table-colgroup').innerHTML),
 				'head': this.tpl($('#tpl-table-thead').innerHTML),
 				'meta': this.tpl($('#tpl-table-row-meta').innerHTML),
 				'data': this.tpl($('#tpl-table-row').innerHTML),
@@ -1226,6 +1485,11 @@ GoAccess.App = {
 		$('nav').classList.remove('hide');
 		$('.container').classList.remove('hide');
 		$('.spinner').classList.add('hide');
+
+		if (GoAccess.AppPrefs['layout'] == 'horizontal') {
+			$('.container').classList.add('container-fluid');
+			$('.container-fluid').classList.remove('container');
+		}
 	},
 
 	renderData: function () {
@@ -1253,41 +1517,7 @@ window.onload = function () {
 		'uiData': window.user_interface,
 		'panelData': window.json_data,
 		'wsConnection': window.connection || null,
+		'prefs': window.html_prefs || {},
 	});
 	GoAccess.App.initialize();
-
-	/*!
-	 * Bootstrap without jQuery v0.6.1 for Bootstrap 3
-	 * By Daniel Davis under MIT License
-	 * https://github.com/tagawa/bootstrap-without-jquery
-	 */
-	function doDropdown(event) {
-		event = event || window.event;
-		var evTarget = event.currentTarget || event.srcElement;
-		evTarget.parentElement.classList.toggle('open');
-		return false;
-	}
-
-	function closeDropdown(event) {
-		event = event || window.event;
-		var evTarget = event.currentTarget || event.srcElement;
-
-		// hacky but FF needs it
-		window.setTimeout(function () {
-			evTarget.parentElement.classList.remove('open');
-		}, 150);
-
-		// Trigger the click event on the target if not opening another menu
-		if (event.relatedTarget && event.relatedTarget.getAttribute('data-toggle') !== 'dropdown') {
-			event.relatedTarget.click();
-		}
-		return false;
-	}
-	var dropdownList = document.querySelectorAll('[data-toggle=dropdown]');
-	for (var k = 0, dropdown, lenk = dropdownList.length; k < lenk; k++) {
-		dropdown = dropdownList[k];
-		dropdown.setAttribute('tabindex', '0'); // Fix to make onblur work in Chrome
-		dropdown.onclick = doDropdown;
-		dropdown.onblur = closeDropdown;
-	}
 };
