@@ -1,5 +1,30 @@
 'use strict';
 
+// This is faster than calculating the exact length of each label.
+// e.g., getComputedTextLength(), slice()...
+function truncate(text, width) {
+	text.each(function () {
+		var parent = this.parentNode, $d3parent = d3.select(parent);
+		var gw = $d3parent.node().getBBox();
+		var x = (Math.min(gw.width, width) / 2) * -1;
+		// adjust wrapper <svg> width
+		if ('svg' == parent.nodeName)
+			$d3parent.attr('width', width).attr('x', x);
+		// wrap <text> within an svg
+		else {
+			$d3parent.insert('svg', function () {
+				return this;
+			}.bind(this))
+			.attr('class', 'wrap-text')
+			.attr('width', width)
+			.attr('x', x)
+			.append(function () {
+				return this;
+			}.bind(this));
+		}
+	});
+}
+
 function AreaChart(dualYaxis) {
 	var opts = {};
 	var margin = {
@@ -8,9 +33,10 @@ function AreaChart(dualYaxis) {
 			bottom : 40,
 			left   : 50
 		},
-		width = 760,
 		height = 170,
-		nTicks = 10;
+		nTicks = 10,
+		padding = 10,
+		width = 760;
 	var labels = { x: 'Unnamed', y0: 'Unnamed', y1: 'Unnamed' };
 	var format = { x: null, y0: null, y1: null};
 
@@ -104,19 +130,6 @@ function AreaChart(dualYaxis) {
 		return height - margin.top - margin.bottom;
 	}
 
-	function trunc(text, width, padding) {
-		text.each(function () {
-			var self = d3.select(this),
-				textLength = self.node().getComputedTextLength(),
-				text = self.text();
-			while (textLength > (width - 2 * padding) && text.length > 0) {
-				text = text.slice(0, -1);
-				self.text(text + '...');
-				textLength = self.node().getComputedTextLength();
-			}
-		})
-	}
-
 	function getXTicks(data) {
 		if (data.length < nTicks)
 			return xScale.domain();
@@ -290,7 +303,8 @@ function AreaChart(dualYaxis) {
 			.attr('d', line)
 			.attr('class', cName)
 			.attr('stroke-dasharray', function (d) {
-				return pathLen(d3.select(this)) + ' ' + pathLen(d3.select(this))
+				var pl = pathLen(d3.select(this));
+				return pl + ' ' + pl;
 			})
 			.attr('stroke-dashoffset', function (d) {
 				return pathLen(d3.select(this))
@@ -300,7 +314,8 @@ function AreaChart(dualYaxis) {
 			.attr('d', line)
 			.transition()
 			.attr('stroke-dasharray', function (d) {
-				return pathLen(d3.select(this)) + ' ' + pathLen(d3.select(this))
+				var pl = pathLen(d3.select(this));
+				return pl + ' ' + pl;
 			})
 			.duration(2000)
 			.attr('stroke-dashoffset', 0);
@@ -370,20 +385,19 @@ function AreaChart(dualYaxis) {
 
 	function addAxis(g, data) {
 		var xTicks = getXTicks(data);
+		var tickDistance = xTicks.length > 1 ? (xScale(xTicks[1]) - xScale(xTicks[0])) : innerW();
+		var labelW = tickDistance - padding;
+
 		// Update the x-axis.
 		g.select('.x.axis')
 			.attr('transform', 'translate(0,' + yScale0.range()[0] + ')')
-			.call(xAxis
-				.tickValues(xTicks)
-			 )
+			.call(xAxis.tickValues(xTicks))
 			.selectAll(".tick text")
-			.call(trunc, (innerW() / xTicks.length), 5);
+			.call(truncate, labelW > 0 ? labelW : innerW());
 
 		// Update the y0-axis.
 		g.select('.y0.axis')
-			.call(yAxis0
-				.tickValues(getYTicks(yScale0))
-			);
+			.call(yAxis0.tickValues(getYTicks(yScale0))	);
 
 		if (!dualYaxis)
 			return;
@@ -391,9 +405,7 @@ function AreaChart(dualYaxis) {
 		// Update the y1-axis.
 		g.select('.y1.axis')
 			.attr('transform', 'translate(' + innerW() + ', 0)')
-			.call(yAxis1
-				.tickValues(getYTicks(yScale1))
-			);
+			.call(yAxis1.tickValues(getYTicks(yScale1)));
 	}
 
 	// Update the X-Y grid.
@@ -569,9 +581,10 @@ function BarChart(dualYaxis) {
 			bottom : 40,
 			left   : 50
 		},
-		width = 760,
 		height = 170,
-		nTicks = 10;
+		nTicks = 10,
+		padding = 10,
+		width = 760;
 	var labels = { x: 'Unnamed', y0: 'Unnamed', y1: 'Unnamed' };
 	var format = { x: null, y0: null, y1: null};
 
@@ -624,21 +637,6 @@ function BarChart(dualYaxis) {
 		.scale(yScale0)
 		.orient('left');
 
-	// The x-accessor for the path generator; xScale ∘ xValue.
-	function X(d) {
-		return xScale(d[0]);
-	}
-
-	// The x-accessor for the path generator; yScale0 yValue0.
-	function Y0(d) {
-		return yScale0(d[1]);
-	}
-
-	// The x-accessor for the path generator; yScale0 yValue0.
-	function Y1(d) {
-		return yScale1(d[2]);
-	}
-
 	function innerW() {
 		return width - margin.left - margin.right;
 	}
@@ -647,19 +645,6 @@ function BarChart(dualYaxis) {
 		return height - margin.top - margin.bottom;
 	}
 
-	function trunc(text, width, padding) {
-		text.each(function () {
-			var self = d3.select(this),
-				textLength = self.node().getComputedTextLength(),
-				text = self.text();
-			while (textLength > (width - 2 * padding) && text.length > 0) {
-				text = text.slice(0, -1);
-				self.text(text + '...');
-				textLength = self.node().getComputedTextLength();
-			}
-		})
-	}
-
 	function getXTicks(data) {
 		if (data.length < nTicks)
 			return xScale.domain();
@@ -692,7 +677,7 @@ function BarChart(dualYaxis) {
 		xScale.domain(data.map(function (d) {
 			return d[0];
 		}))
-		.rangeRoundBands([0, innerW()], .1);
+		.rangeBands([0, innerW()], .1);
 
 		// Update the y-scale.
 		yScale0.domain([0, d3.max(data, function (d) {
@@ -806,10 +791,6 @@ function BarChart(dualYaxis) {
 			.attr('class', 'indicator');
 	}
 
-	function addBar() {
-
-	}
-
 	// Update the area path and lines.
 	function addBars(g, data) {
 		var bars = g.select('g.bars.y0').selectAll('rect.bar')
@@ -864,20 +845,19 @@ function BarChart(dualYaxis) {
 
 	function addAxis(g, data) {
 		var xTicks = getXTicks(data);
+		var tickDistance = xTicks.length > 1 ? (xScale(xTicks[1]) - xScale(xTicks[0])) : innerW();
+		var labelW = tickDistance - padding;
+
 		// Update the x-axis.
 		g.select('.x.axis')
 			.attr('transform', 'translate(0,' + yScale0.range()[0] + ')')
-			.call(xAxis
-				.tickValues(xTicks)
-			 )
+			.call(xAxis.tickValues(xTicks))
 			.selectAll(".tick text")
-			.call(trunc, (innerW() / xTicks.length), 5);
+			.call(truncate, labelW > 0 ? labelW : innerW());
 
 		// Update the y0-axis.
 		g.select('.y0.axis')
-			.call(yAxis0
-				.tickValues(getYTicks(yScale0))
-			);
+			.call(yAxis0.tickValues(getYTicks(yScale0)));
 
 		if (!dualYaxis)
 			return;
@@ -885,9 +865,7 @@ function BarChart(dualYaxis) {
 		// Update the y1-axis.
 		g.select('.y1.axis')
 			.attr('transform', 'translate(' + innerW() + ', 0)')
-			.call(yAxis1
-				.tickValues(getYTicks(yScale1))
-			);
+			.call(yAxis1.tickValues(getYTicks(yScale1)));
 	}
 
 	// Update the X-Y grid.
